//   genpres4_stag.c
//      routines for generation/simulation of staggered-entry occupancy model
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <process.h>
#include "rngs.h"

#define VER "<110914.0943>"

FILE *fopen(), *fp, *fp2, *fp3;
int num_srvys,nout;

void bin2str(int x, int n, char *s) {/* converts n digit base 2 number, 'x' */
   int i,j; strcpy(s,"");           //   and stores in string 's'
   for (i=0,j=1<<(n-1); j>0; j>>=1) 
      if (((x&j)&&1)>0) strcat(s,"1"); else strcat(s,"0"); 
}
//  forward declaration of staggered entry procedure... see description below
void stag_ent(double n, int t, long ch, double psi, double *e, double *d, double *p, int entered);
    
void stag_depart(double n, int t, long ch, double psi, double *e, double *d, double *p) {
/*    staggered departure procedure , splits group of sites into two groups: departed and not-departed
 *                                then calls entry procedure (stag_ent) for each group in survey t+1
 * */
    double ndepart,notdepart; char s[256];
    ndepart=n*d[t]; notdepart=n-ndepart;
    bin2str(ch, num_srvys,s); 
                                           //  sites which departed can have no more detections, so
    fprintf(fp,"%s %18.12f;\n",s,ndepart); nout++; // print history for departed sites
    
    stag_ent(notdepart, t+1,ch,psi,e,d,p,1); // call entry procedure in survey t+1 for not-departed sites
}

void stag_cap(double n, int t, long ch, double psi, double *e, double *d, double *p, int entered) {
/*    staggered capture procedure , splits group of sites into two groups: captured and not-captured
 *                                then calls departure procedure (stag_depart) for each group
 */
    double ncap,notcap; int k; char s[256];
    if (entered) {
        ncap=n*p[t]; // number captured (sites with >= 1 individual detected) 
        notcap=n-ncap;       // number not captured (sites with 0 detections)
        k=(1<<(num_srvys-t-1));   //   binary representation of '1' for survey 't'
        if (t<(num_srvys-1)) {
            //       call departure procedure with current detection history
            stag_depart(ncap  , t,ch|k,psi,e,d,p);//  and '1' for detection in survey t
            stag_depart(notcap, t,ch  ,psi,e,d,p);//  and '0' for non-detection 
        }
        else {//         if last survey, convert binary to string and print detection history
            bin2str(ch|k, num_srvys,s); fprintf(fp,"%s %18.12f;\n",s,ncap); nout++;
            bin2str(ch  , num_srvys,s); fprintf(fp,"%s %18.12f;\n",s,notcap); nout++;
        }
    }
    else//       not entered yet... detections impossible, so just see if they enter in next srvy
        if (t<(num_srvys-1)) 
            stag_ent(n  , t+1,ch,psi,e,d,p,0);
        else {  //  not entered and last survey... print detection history
            bin2str(ch, num_srvys,s); fprintf(fp,"%s %18.12f;\n",s,n); nout++; // print history
        }
}

void stag_ent(double n, int t, long ch, double psi, double *e, double *d, double *p, int entered) {
/*    staggered entry procedure , splits group of sites into two groups: entered and non-entered
 *                                then calls detection procedure (stag_cap) for each group
 * */
    double nent,notent;
    if (!entered) {
        nent=n*e[t]; //  number entered = binomial (N=n sites, p=entry prob(srvy))
        notent=n-nent;       //  not entered = n minus number entered
        
        stag_cap(nent  , t,ch,psi,e,d,p,1);  //   last arg. defines entry state (1=entered,
        stag_cap(notent, t,ch,psi,e,d,p,0);  //                            0=not entered yet)
    }
    else
        stag_cap(n     , t,ch,psi,e,d,p,1);
}

int do_stag_entry_model_sim(int argc, char **argv, int SIMTYPE) {
    /* procedure to simulate Kendall staggered-entry model
     * (single-season model where sites can become occupied after the first
     * survey, and become not occupied before the last survey.  Sites may 
     * not transition back and forth between occupied and not-occupied, but
     * may start out unoccupied, transition to occupied, then transition to
     * unoccupied.  One restriction: each site must become occupied by the
       last survey.)
     */
    int site,srvy,j,  //  indices
          k,ch,     //  binary representation of detection history
          occ,      //  logical: occupied or unoccupied
          nsites,       //     number of sites
          entered,departed,  //  logical indicators 
          sptr;      // 
    char buf[256],  //  buffer to stor input lines
          *s,       //  temp string pointer
          t[256];   //   temp string
    double *p,    // detection prob. vector
          psi,    // occupancy rate
          *e,     // entry probability vector
          *d,     // departure probability vector
          num_occ,num_unocc,  //  number occupied, number unoccupied
          ns; 
    
    //    get input paramsters from command line...
    
    num_srvys=atoi(argv[1]); nsites=atoi(argv[2]); 
    printf("num_srvys,nsites=%d,%d\n",num_srvys,nsites);
    psi=atof(argv[3]); printf("psi: %4.2f\n",psi);
    j=3;  //  command-line argument index
    //   allocate memory for parameter vectors
    e=(double*)malloc(num_srvys*sizeof(double)); 
    d=(double*)malloc(num_srvys*sizeof(double)); 
    p=(double*)malloc(num_srvys*sizeof(double));
    
    for (srvy=0; srvy<num_srvys; srvy++) p[srvy]=atof(argv[++j]);  //  get detection prob. vector and print
    printf("p:"); for (srvy=0; srvy<num_srvys; srvy++) printf(" %4.2f",p[srvy]); printf("\n");
    
    //   get entry prob. vector
    for (srvy=0; srvy<(num_srvys-1); srvy++) e[srvy]=atof(argv[++j]); e[num_srvys-1]=1;  //  must enter by last survey
    printf("e:"); for (srvy=0; srvy<(num_srvys-1); srvy++) printf(" %4.2f",e[srvy]); printf("\n");
    
    //   get departure prob. vector
    for (srvy=0; srvy<(num_srvys-1); srvy++) d[srvy]=atof(argv[++j]); 
    printf("d:"); for (srvy=0; srvy<(num_srvys-1); srvy++) printf(" %4.2f",d[srvy]); printf("\n\n");
    
    printf("opening genpres.inp...\n");         //  open output file (named genpres.inp
    if ((fp=fopen("genpres.inp","w"))==NULL) {  //    to be used as an input file to
        printf("error opening genpres.inp\n");  //    program MARK).
        exit(1);
    }
    
    /*  Two ways of simulating data..
     *     1)  start with N sites, then apply binomial function to split into
     *         occupied and unoccupied sites.  Then split occupied sites into
     *         detected and undetected sites.  Keep splitting each group of
     *         sites recursively until num_surveys is reached.
     * 
     *     2)  Follow each site and generate random number and compare to psi
     *         to determine if site is occupied.  If occupied, generate rand. numbers and
     *          compare to detection probs to determine if species is detected at each survey.
     */
    if (SIMTYPE!=2) { 
        fprintf(stderr,"simulating by binary tree...\n");
        num_occ   = (double)nsites*psi; //  split sites into occupied (num_occ) and
        num_unocc = (double)nsites - num_occ;           //   unoccupied (num_unocc) sites
        printf("num_occ=%f\n",num_occ);
        
        //    now, call recursive staggered-entry procedure to generate detection histories
        stag_ent(num_occ  ,  // first argument = number of occupied sites
                 0,          //  2nd argument = starting survey number minus 1
                 0,          //  3rd argument = starting detection history in binary notation
                 psi,        //   occupancy rate (prob. site will be occupied at some point)
                 e,          //   entry prob. vector  (prob unoccupied site becomes occupied)
                 d,          //   departure prob. vector (prob occupied site becomes unoccupied)
                 p,          //   detection prob. vector (prob. of detection at occupied site)
                 0);  // last parm = entry state : 0=not entered yet, 1=entered)
        
        for (srvy=0; srvy<num_srvys; srvy++) fprintf(fp,"0");  //  print history for unoccupied sites
        fprintf(fp," %18.12f;\n",num_unocc); nout++;
    }
    else { 
        fprintf(stderr,"simulating by individual...\n");
        
        for (site=0; site<nsites; site++) {                            //  
            
            occ=(Random()<psi);   // draw rand. number and compare to psi to determine if site is occupied
            entered=departed=ch=0;  //  initialize entry,departure,and detection to zero
            
            for (srvy=0,k=(1<<(num_srvys-1)); srvy<num_srvys; srvy++,k>>=1) { //  loop through each survey
                                                                  //       using k as binary indicator of a
                if (occ) {       //  if site is occupied...                detection in survey, 'srvy'
                    
                                                                //  if site hasn't been occupied yet
                                                               //    draw rand. number to see if site 
                    if (!entered) entered=(Random()<e[srvy]); //     becomes occupied this survey
                    
                    if (entered && !departed) {  // if site has entered occupied state and not departed yet...
                        if (Random()<p[srvy]) ch|=k;  //   test for detection and set detection history (ch)
                        departed=(Random()<d[srvy]);  //   draw rand. number to test if sites departs occupied state
                    }
                }
            }
            //    convert detection history (ch) from binary to a string and print
            bin2str(ch  , //   binary representation of detection history (eg., 44=101100)
                   num_srvys,  //  number of surveys
                   t);         //  output detection history string (eg., '101100')
            fprintf(fp,"%s %18.12f;\n",t,1.0); nout++;
        }
    }
    fclose(fp);
    //     open and read output file to count number of lines
    fp=fopen("genpres.inp","r"); nout=0;
    while (fgets(buf, 256, fp)) { 
        s=strtok(buf," "); s=strtok(NULL," "); ns=atof(s); if (ns>0) nout++;
    }
    fclose(fp);
    //    create 2nd output file (genpres.pao) in format acceptable to PRESENCE
    fp=fopen("genpres.inp","r");   //  1st output file
    fp2=fopen("genpres.pao","w");  //  2nd output file: needs TABS between surveys
    fprintf(fp2,"%d%c%d\n",nout,9,num_srvys);//   and init. line w/ number of sites and surveys
    while (fgets(buf, 256, fp)) { 
        s=strtok(buf," "); strcpy(t,s); //  copy line and insert TAB characters
        for (srvy=0; srvy<num_srvys; srvy++) { t[srvy*2]=s[srvy]; t[srvy*2+1]='\t'; } t[num_srvys*2]='\0'; 
        s=strtok(NULL," "); sptr=strchr(s,';')-s; s[sptr]='\0'; 
        ns=atof(s);                          //  last field in line is frequency count
        if (ns>0) fprintf(fp2,"%s%s\n",t,s); 
    }
                                             //  after detection histories, add zeros for 
    fprintf(fp2,"0\n0\n"); //   number of site covariates and srvy covariates 
    fprintf(fp2,"%d\n",num_srvys);  //  add number of srvys/season
    fprintf(fp2,"simulated data from genpres3\n");  // add title
    fclose(fp2); fclose(fp);
    printf("end of program(genpres3.exe)\n"); exit(0);
}

#ifndef NOTSTANDALONE
int main(int argc, char **argv) {
  int SIMTYPE=0; nout=0; 
  if (argc<7) { 
    printf("usage: genpres4_stag T N psi p e d [SIMTYPE=2]\n");
	printf("   where T=number of surveys\n");
	printf("         N=number of sites\n");
	printf("         psi=prob. occupancy\n");
	printf("         p=vector of detection probs. (T values)\n");
	printf("         e=vector of entry probs. (T-1 values)\n");
	printf("         d=vector of departure probs. (T-1 values)\n");
	printf("         SIMTYPE=2 - option to simulate by site.\n");
	printf("    (eg., genpres4_stag 4 100 .8 .6 .6 .6 .6 .1 .1 .1 .2 .2 .2)\n");
	exit(1);
  }
  if (strstr(argv[argc-1],"SIMTYPE")!=NULL) SIMTYPE=2;
  do_stag_entry_model_sim(argc,argv,SIMTYPE);
}
#endif